﻿using UnityEngine;
using System.Collections;

public class BGMController : MonoBehaviour {
    public AudioClip Music;
    public float WaitBeforePlaying = 0;
	public string[] AudioLevels = new string[] {"BGMVolume","MasterVolume"};
    //Loop back to this point
    public float LoopBackTo = 0;
	public bool KeepAlive = true;
    //Time from the END of the track to loop from (crossfade)
    public float LoopFromEnd = 0;
    public bool firstPlay;
    public AudioSource CurrentlyPlaying, NextPlaying;
    void Start()
    {
        if (WaitBeforePlaying > 0) Invoke("PlayBGM", WaitBeforePlaying);
        else Invoke("PlayBGM", 0.01f);
    }
    public void Restart()
    {
        Debug.Log("RESTARTING");
        if (CurrentlyPlaying != null)
        {
            if (CurrentlyPlaying.gameObject != this.gameObject)
            {
                Debug.Log("RESTART, DESTROYING CURRENTLY PLAYING");
                Destroy(CurrentlyPlaying.gameObject);
                CurrentlyPlaying = null;
                NextPlaying = null;
            }
            else
            {
                this.GetComponent<AudioSource>().Stop();
            }
        }
        if (NextPlaying != null)
        {
            Destroy(NextPlaying.gameObject);
            NextPlaying = null;
        }
        PlayBGM();
    }
    void PlayBGM()
    {
        if (CurrentlyPlaying == null)
        {
            CurrentlyPlaying = this.GetComponent<AudioSource>();
            CurrentlyPlaying.Stop();
        }
        if (NextPlaying == null)
        {
            GameObject p = new GameObject("AudioSource");
            NextPlaying = p.AddComponent<AudioSource>();
            NextPlaying.playOnAwake = false;
            NextPlaying.time = LoopBackTo;
            p.transform.parent = this.transform;
            p.transform.localPosition = Vector3.zero; 
        }

        UpdateVolume();
        CurrentlyPlaying.enabled = true;
            CurrentlyPlaying.clip = Music;
            CurrentlyPlaying.Play();
            CurrentlyPlaying.loop = false; 
        NextPlaying.clip = Music;
        NextPlaying.Stop();
        
        NextPlaying.loop = false;
        firstPlay = true;
    }
    public void UpdateVolume()
    {
        float multiplier = 1;
        foreach (string pref in AudioLevels)
        {
            multiplier *= PlayerPrefs.GetFloat(pref, 0.5f);
        }
        if (CurrentlyPlaying != null)
        {
            CurrentlyPlaying.volume = multiplier;
        }
        if (NextPlaying != null)
        {
            NextPlaying.volume = multiplier;
        }
    }
	void Awake() {
		if (FindObjectOfType (typeof(BGMController)) != this) {
						Destroy (this.gameObject);
				}
		if(KeepAlive)
		DontDestroyOnLoad (this.gameObject);
	}
    void LateUpdate()
    {
        if (CurrentlyPlaying != null)
        {
        if (firstPlay && CurrentlyPlaying.time > LoopBackTo) firstPlay = false;
            if (CurrentlyPlaying.time > Music.length - LoopFromEnd)// || (!firstPlay && CurrentlyPlaying.time <= LoopBackTo))
            {
                NextPlaying.Play();
                CurrentlyPlaying.Stop();
                var p = CurrentlyPlaying;
                CurrentlyPlaying = NextPlaying;
                NextPlaying = p;
                CurrentlyPlaying.time = LoopBackTo;
                Debug.Log("SWITCH!");
                //if (!this.audio.isPlaying) this.audio.Play();
                //this.audio.time = LoopBackTo;
            }
        }
    }
    /*
	public void ChangeClip(AudioClip c) {
		this.audio.clip = c;
		this.audio.Play ();
		this.audio.loop = false;
	}
	// Use this for initialization
	void Start () {
        if (WaitBeforePlaying > 0) Invoke("PlayBGM", WaitBeforePlaying);
        else PlayBGM();
    }
    void PlayBGM() {
		UpdateVolume();
        //this.audio.volume = PlayerPrefs.GetFloat("BGMVolume",0.2f);
        this.audio.clip = Music;
        this.audio.Play();
        this.audio.loop = true;
        firstPlay = true;
	}
	public void UpdateVolume() {
		float multiplier = 1;
		foreach(string pref in AudioLevels) {
			multiplier *= PlayerPrefs.GetFloat(pref,0.5f);
		}
		this.audio.volume = multiplier;
	}
	// Update is called once per frame
	void LateUpdate () {
        if (firstPlay && this.audio.time > LoopBackTo) firstPlay = false;
        if (this.audio.time > Music.length - LoopFromEnd || (!firstPlay && this.audio.time <= LoopBackTo))
        {
            if(!this.audio.isPlaying) this.audio.Play();
            this.audio.time = LoopBackTo;
        }
	}*/

    internal void Play()
    {
        CurrentlyPlaying.Play();
    }

    internal void Stop()
    {
        CurrentlyPlaying.Stop();
    }
}
