﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine.Events;

public class PathMovement : MonoBehaviour {
    public List<Vector2> Path;
    public float Speed = 1;
    public UnityEvent EndOfPath;
    public enum RepeatType
    {
        Loop, Reverse, Snap
    }
    public void Reset()
    { 
        t = 0;
        this.transform.position = StartPosition;
    }
    public RepeatType Repeat = RepeatType.Loop;
    public void OnDrawGizmos()
    {
        Vector2 sp = StartPosition == Vector3.zero ? this.transform.position : StartPosition;
        //Gizmos.matrix = this.transform.localToWorldMatrix;
        if (Path != null && Path.Count > 0)
        { 
            
            for (int i = 1; i < Path.Count; i++)
            {
                Gizmos.DrawLine(sp + Path[i - 1], sp+Path[i]);
            }
            if (Repeat == RepeatType.Loop)
            {
                Gizmos.DrawLine(sp + Path[Path.Count - 1], sp + Path[0]);
            }
        }
    }
    private float t = 0;
    private List<Vector2> ActualPath;
    private Vector3 StartPosition;
    void Start()
    {
        StartPosition = this.transform.localPosition;
        ActualPath = new List<Vector2>();
        ActualPath.AddRange(Path);
        if (Repeat == RepeatType.Reverse)
        {
            ActualPath.AddRange(Path.Skip(1).Take(Path.Count - 2).Reverse().ToList());
        }
        lastPosition = this.transform.position;
    }
	// Update is called once per frame
    private Vector3 lastPosition;
	void Update () {
        t += Time.deltaTime * Speed;
        this.transform.localPosition = StartPosition + GetPosition(t);
        float dx = this.transform.position.x - lastPosition.x;
        this.transform.localScale = new Vector3(dx < 0 ? 1 : -1, this.transform.localScale.y, this.transform.localScale.z);
        lastPosition = this.transform.position;
	}

    private Vector3 GetPosition(float t)
    {
        t %= ActualPath.Count;

        int point = Mathf.FloorToInt(t);
        int endpoint = point + 1;
        if (endpoint >= ActualPath.Count)
        {
            endpoint = 0;
            EndOfPath.Invoke();
            if (Repeat == RepeatType.Snap)
            {

                t = 0;
                return ActualPath[endpoint];
            }
        }
        float pt = t - point;

        return Vector2.Lerp(ActualPath[point], ActualPath[endpoint], pt);

    }

}
