﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace com.geeitsomelaldy.platforming
{
    public class BaseCollider:MonoBehaviour
    {
        private Vector2 tl, tr, bl, br;
        public RaycastHit2D hit { get; protected set; }
        [System.Serializable]
        public class BoundsState
        {
            public string Name;
            public Rect HitBounds;
            public Color debugColor;
        }
        public List<BoundsState> Bounds;
        private string _currentBoundsState = "";
        public string CurrentBoundsState
        {
            get
            {
                return _currentBoundsState;
            }
            set
            {
                if (value != _currentBoundsState)
                { 
                    _currentBoundsState = value;
                    _currentState = null;
                    SetCorners(); 
                    this.BroadcastMessage("CollidersDirty", SendMessageOptions.DontRequireReceiver);
                }
            }
        }
        private BoundsState _currentState; 
        public Rect HitBounds
        {
            get
            {
                if (_currentState == null)
                {
                    _currentState = Bounds.FirstOrDefault(x => x.Name == CurrentBoundsState);
                    if (_currentState == null) _currentState = Bounds.FirstOrDefault();
                } 
                return _currentState.HitBounds;
            }
        }
        void OnDrawGizmos()
        {
            Gizmos.color = Color.white;
            foreach (var b in Bounds)
            {
                Gizmos.color = b.Name == CurrentBoundsState?Color.white: b.debugColor;
                Gizmos.DrawWireCube(this.transform.position + (Vector3)b.HitBounds.position, b.HitBounds.size * 2);
            }
        }
        public bool HasHit(LayerMask mask)
        {
            Vector2 pos = (Vector2)this.transform.position;
            //hit = Physics2D.BoxCast(pos, HitBounds.size, 0, Vector2.up, 0, mask);
            //Debug.DrawLine(pos + tl, pos + tl + (Vector2.right * HitBounds.size.x * 2), Color.magenta, 1);
            hit = Physics2D.Raycast(pos + tl, Vector2.right, HitBounds.size.x * 2,mask);
            if (hit.collider != null)
            {
                Debug.Log("HIT 1");
                Debug.DrawLine(pos + tl, hit.point, Color.red, 2);
                return true;
            }
            //Debug.DrawLine(pos + bl, pos + bl + (Vector2.right * HitBounds.size.x * 2), Color.yellow, 1);
            hit = Physics2D.Raycast(pos + bl, Vector2.right, HitBounds.size.x * 2, mask);
            if (hit.collider != null)
            {
                Debug.DrawLine(pos + bl, hit.point, Color.red, 2);
                return true;
            }
            //Debug.DrawLine(pos + tr, pos + tr + (-Vector2.up * HitBounds.size.y * 2), Color.red, 1);
            hit = Physics2D.Raycast(pos + tr, -Vector2.up, HitBounds.size.y * 2, mask);
            if (hit.collider != null)
            {
                Debug.DrawLine(pos + tr, hit.point, Color.red, 2);
                return true;
            }
            //Debug.DrawLine(pos + tl, pos + tl + (-Vector2.up * HitBounds.size.y * 2), Color.blue, 1);
            hit = Physics2D.Raycast(pos + tl, -Vector2.up, HitBounds.size.y * 2, mask);
            if (hit.collider != null)
            {
                Debug.DrawLine(pos + tl, hit.point, Color.red, 2);
                return true;
            }
            /*if (hit.collider != null) return true;
            hit = Physics2D.Linecast(pos + bl, pos + br, mask);
            if (hit.collider != null) return true;
            hit = Physics2D.Linecast(pos + tl, pos + bl, mask);
            if (hit.collider != null) return true;
            hit = Physics2D.Linecast(pos + tr, pos + br, mask);
            if (hit.collider != null) return true;
            hit = Physics2D.Linecast(pos + tl, pos + tr, mask);
            if (hit.collider != null) return true;*/

            return false;
        }
        void Awake()
        {
            SetCorners();
        }

        void SetCorners() {
            tl = new Vector2(-HitBounds.xMax, HitBounds.yMax);
            tr = new Vector2(HitBounds.xMax, HitBounds.yMax);
            bl = new Vector2(-HitBounds.xMax, -HitBounds.yMax);
            br = new Vector2(HitBounds.xMax, -HitBounds.yMax);
        }
    }
}
