﻿using UnityEngine;
using System.Collections;

namespace com.geeitsomelaldy.platforming
{
    public class PlatformerMecanimAnimator : MonoBehaviour
    {
        public Animator Animator;
        [System.Serializable]
        public class MecanimAnimatorParameters
        {
            public string HorizontalSpeed = "Horizontal";
            public string VerticalSpeed = "Vertical";
            public string IsGrounded = "Grounded";
            public string IsSkidding = "Skidding";
            public string IsCrouched = "Crouched"; 
        }
        public MecanimAnimatorParameters AnimatorParameters;
        private PlayerPlatformController playerPlatformController;
        // Use this for initialization
        void Start()
        {
            playerPlatformController = this.GetComponent<PlayerPlatformController>();
            playerPlatformController.Events.OnGroundedChanged.AddListener((b) =>
            {
                Animator.SetBool(AnimatorParameters.IsGrounded, b);
            });
            playerPlatformController.Events.OnSkid.AddListener((b) =>
            {
                Animator.SetBool(AnimatorParameters.IsSkidding, b);
            });
            playerPlatformController.Events.OnVelocityChanged.AddListener((v) =>
            {
                if (v.x < 0) Animator.transform.localScale = new Vector3(-1, 1, 1);
                if (v.x > 0) Animator.transform.localScale = new Vector3(1, 1, 1);
                Animator.SetFloat(AnimatorParameters.HorizontalSpeed, Mathf.Abs(v.x));
                Animator.SetFloat(AnimatorParameters.VerticalSpeed, v.y);
            });
            var input = this.GetComponent<PlayerInput>();
            if (input != null)
            {
                input.OnCrouchBegin.AddListener(() =>
                {
                    Animator.SetBool(AnimatorParameters.IsCrouched, true);
                });
                input.OnCrouchEnd.AddListener(() =>
                {
                    Animator.SetBool(AnimatorParameters.IsCrouched, false);

                });
            }
        } 
    }
}