﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;
namespace com.geeitsomelaldy.platforming
{
    [RequireComponent(typeof(com.geeitsomelaldy.platforming.PlayerPlatformController))]
    public class PlayerInput : MonoBehaviour
    {
        public float HorizontalAcceleration = 1;
        public float HorizontalAirAcceleration = 1;
        public float HorizontalSkidDeceleration = 1;
        public bool useDigitalControls = false;
        public float JumpStrength = 10;
        public float JumpSpeedAfterRelease = 10;
        public float JumpLongPressTime = 0.2f;

        public string HorizontalAxis = "Horizontal";
        public string VerticalAxis = "Vertical";
        public string JumpButton = "Jump";
        public UnityEvent OnCrouchBegin, OnCrouchEnd;
        public bool isCrouched;
        private PlayerPlatformController _platformController;
        private PlayerPlatformController platformController
        {
            get
            {
                if (_platformController == null) _platformController = this.GetComponent<PlayerPlatformController>();
                return _platformController;
            }
        }
        public float horizontal, vertical;
        bool isSkidding = false;
        // Update is called once per frame
        void Update()
        {
            horizontal = useDigitalControls?Input.GetAxisRaw(HorizontalAxis) : Input.GetAxis(HorizontalAxis);
            vertical = useDigitalControls ? Input.GetAxisRaw(VerticalAxis) : Input.GetAxis(VerticalAxis);

            var willCrouch = (platformController.isGrounded && vertical < 0);
            if (willCrouch && !isCrouched)
            {
                isCrouched = true;
                OnCrouchBegin.Invoke();
            }
            else if (!willCrouch && isCrouched)
            {
                isCrouched = false;
                OnCrouchEnd.Invoke();

            }
            if (isCrouched) horizontal = 0;
           
            if (horizontal != 0 && Mathf.Sign(horizontal) != Mathf.Sign(platformController.Velocity.x))
            {
                if (!isSkidding)
                { 
                    isSkidding = true;
                    platformController.Events.OnSkid.Invoke(true);
                }
            }
            else
            {
                if (isSkidding)
                { 
                    isSkidding = false;
                    platformController.Events.OnSkid.Invoke(false);
                }
            }
            if (!isJumping && platformController.isGrounded && Input.GetButton(JumpButton))
            {
                StopCoroutine("JumpLongPress");
                StartCoroutine(JumpLongPress());
            }
            if (horizontal != 0)
            {
                platformController.applyFriction = false;
                platformController.Velocity.x += 
                    Time.deltaTime *
                    (isSkidding ? HorizontalSkidDeceleration : platformController.isGrounded ? HorizontalAcceleration : HorizontalAirAcceleration) * horizontal;
            }
            else
            {
                platformController.applyFriction = true;
            }
        }
        bool isJumping = false;
        IEnumerator JumpLongPress()
        {  
                for (float t = 0; t < JumpLongPressTime; t += Time.deltaTime)
                {
                    platformController.Velocity.y = JumpStrength;
                    yield return null;
                    if (
                        platformController.Velocity.y <= 0 || 
                        platformController.isGrounded || 
                        !Input.GetButton(JumpButton)
                        )
                    {
                        break;
                    } 
                }
                platformController.Velocity.y = JumpSpeedAfterRelease;  
        }
    }
}