﻿using UnityEngine;
using System.Collections;
using UnityEngine.Events;

namespace com.geeitsomelaldy.platforming
{
    [System.Serializable]
    public class PlayerPlatformerEvents
    {
        public Vector2Event OnVelocityChanged;
        public BooleanEvent OnGroundedChanged, OnSkid;
        public UnityEvent OnCrushed;
    }
    public class PlayerPlatformController : LaldyPlatformBase
    {
        public float Gravity = 9.81f;
        public float Friction = 10;
        public void SetFriction(float n)
        {
            Friction = n;
        }
        public bool UseFriction = true;
        /// <summary>
        /// If the character lands on a platform on this layer, 
        /// it'll reparent the character to the game object
        /// </summary>
        public LayerMask MoveableGroundLayers;
        /// <summary>
        /// The initial parent, can be null. Used when you transition to air
        /// </summary>
        private Transform DefaultParent;
        /// <summary>
        /// The direction the character is facing (-1 or 1)
        /// </summary>
        public int Direction { get; protected set; }
        /// <summary>
        /// Character is on the ground
        /// </summary>

        public bool isGrounded { get; protected set; }
        public bool applyFriction;
        public PlayerPlatformerEvents Events;
        public void Start()
        {
            DefaultParent = this.transform.parent;
        }
        public void OnEnable()
        {
            Velocity = Vector2.zero;
        }
        public void Update()
        {

            if (Velocity.x < 0) Direction = -1;
            if (Velocity.x > 0) Direction = 1;
            if (applyFriction && Velocity.x != 0) //Apply Friction
            {
                if (UseFriction)
                {
                    Velocity.x -= Direction * Friction * Time.deltaTime;
                    //come to a complete stop if we've flipped signs
                    if ((Direction > 0 && Velocity.x < 0) || (Direction < 0 && Velocity.x > 0))
                    {
                        Velocity.x = 0;
                    }
                }
                else
                {
                    Velocity.x = 0;
                }
            }
 
            if (!isGrounded) Velocity.y -= Gravity * Time.deltaTime ;
            this.Events.OnVelocityChanged.Invoke(Velocity);
        }

        protected override void OnGroundHit(RaycastHit2D hit)
        {
            if (MoveableGroundLayers.Contains(hit.collider.gameObject.layer))
            {
                this.transform.parent = hit.collider.transform;
            }
            else
            {
                this.transform.parent = DefaultParent;
            }
            Velocity.y = 0;
            isGrounded = true;
            this.Events.OnGroundedChanged.Invoke(true);
            this.transform.position = new Vector3(
                this.transform.position.x,
                hit.point.y + this.ColliderSize.y - this.ColliderOffset.y,
                this.transform.position.z
            );
        }

        protected override void OnCeilingHit(RaycastHit2D hit)
        {
            Velocity.y = 0;
            this.transform.position = new Vector3(
                this.transform.position.x,
                hit.point.y - this.ColliderSize.y + this.ColliderOffset.y,
                this.transform.position.z
            );
        }

        protected override void OnWallHit(RaycastHit2D hit, int CollisionDirection)
        { 
            this.transform.position = new Vector3(
                hit.point.x + (CollisionDirection * this.ColliderSize.x) + this.ColliderOffset.x, 
                this.transform.position.y, 
                this.transform.position.z);
            Velocity.x = 0;
        }

        protected override void OnTransitionToAir()
        {
            this.Events.OnGroundedChanged.Invoke(false);
            isGrounded = false;
        }

        protected override void OnCrushed()
        {
            this.Events.OnCrushed.Invoke();
        }
    }
}